/*
 * Project name:
     Light Click
 * Copyright:
     (c) Mikroelektronika, 2013.
 * Revision History:
     20130417:
       - initial release (VP);
     20130626:
       - modified for EasyPIC v7 (MV);
 * Description:
     This program demonstrates measuring of the Light.
     Microcontroller take the data about light intensity which is measured on the
     Light Click.
     Microcontrollers reads data and displaying on the LCD.

  * Test configuration:
  
     MCU:             ATmega32
                      http://www.atmel.com/Images/doc2503.pdf
     Dev.Board:       EasyAVRv7
                      ac:Easy_AVR_v7
                      http://www.mikroe.com/easyavr/
     Oscillator:      Oscillator Frequency 8.0000 MHz, 8.0000 MHz Crystal
     ext. modules:    Light Click
                      ac:Light_Click
                      http://www.mikroe.com/click/light/
                      LCD 2x16 characters
                      ac:LCD_2x16ch
                      http://www.mikroe.com/store/components/displays/
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/mikroc/avr/
 * NOTES:

     - Plug in LCD 2x16 chars in it's socket on the Development Board
     - Plug in the Light Click in mikroBUS socket 1 on the Development Board
     - Turn on BCK line (Back light on the LCD) at SW3.1
     - Turn on SW5.4, SW5.5, SW5.6
 */

// Light Click connections
sbit Light_CS at PORTA5_bit;
sbit Light_CS_Dir at DDA5_bit;
// End Light Click connections

// Lcd module connections
sbit LCD_RS at PORTA2_bit;
sbit LCD_EN at PORTD6_bit;
sbit LCD_D4 at PORTC4_bit;
sbit LCD_D5 at PORTC5_bit;
sbit LCD_D6 at PORTC6_bit;
sbit LCD_D7 at PORTC7_bit;

sbit LCD_RS_Direction at DDA2_bit;
sbit LCD_EN_Direction at DDD6_bit;
sbit LCD_D4_Direction at DDC4_bit;
sbit LCD_D5_Direction at DDC5_bit;
sbit LCD_D6_Direction at DDC6_bit;
sbit LCD_D7_Direction at DDC7_bit;
// End Lcd module connections

char out[24];
char txt0m[]="Light:";
float  value;

// Display Light Intensity
unsigned int Display_Light_Intensity(unsigned int Light_Intensity){
char text[10], temp[3];

  temp[0] = ' ';
  temp[1] = '%';
  temp[2] = 0;

  // Divide the light intensity value to slace the percent value
  Light_Intensity = Light_Intensity / 40;

  // If the light intensity is more than 100 percent
  if (Light_Intensity >= 100)
    Light_Intensity = 100;

  // Convert light intensity value to a string
  WordToStr(Light_Intensity, text);
  strcat(text, temp);

  // Display light intensity value on the Lcd
  LCD_Out(2, 1, text);
}

// Read light intensity
unsigned int Light_Read() {
  volatile unsigned short tmp1 = 0 , tmp = 0  ;
  volatile unsigned int i = 0, tmp2=0, tmp3=0, avrg = 0;
  short buffer;

  // Get light intensity value from the SPI bus
  for(i=0; i<8; i++ ){
    Light_CS = 0;                 // Select Light click module
    tmp = SPI1_Read(buffer);      // Get first 8 bits of ADC value
    tmp1 = SPI1_Read(buffer);     // Get second 8 bits of ADC value
    Light_CS = 1;                  // Deselect Light click module

    // Format the acquired value
    tmp2 = ((tmp & 0x1F) << 8);
    tmp3 = ((tmp2 | tmp1) >> 1);

    // Sum the acquired values
    avrg += tmp3;
  }

  // Calculate the average value
  avrg /= 8;

  return avrg;
}

void main() {

  Light_CS_Dir = 1;                    // Set CS pin as output
  Light_CS = 1;                        // Deselect Light Click module

  LCD_Init();
  Lcd_Cmd(_LCD_CLEAR);                 // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);            // Cursor off
  Lcd_Out(1,1, "Light Intensity :" );  // Write text in first row

  SPI1_Init();                         // Initialize SPI module

  while(1){
    value = Light_Read();              // Store light intensity value
    Display_Light_Intensity(value);    // Display stored light intensity value
  }
}